#include <iostream>
#include <vector>
#include <cmath>
#include <algorithm>
using namespace std;

vector<unsigned> create( int size, char mode = '-' ) {  // заполнить в указанном порядке
   vector<unsigned> vect = vector<unsigned>( size );
   unsigned j = 0;
   if( '?'  == mode ) srand( (unsigned int)time( NULL ) );
   for( vector<unsigned>::iterator i = vect.begin(); i != vect.end(); i++ )
      switch( mode ) {
         case '+' : 
            *i = ++j;
            break;
         case '?' :
            *i = nearbyint( (double)rand() / RAND_MAX * ( size - 1 ) ) + 1;
            break;
         case '-' :
         default  :
            *i = size--;       
            break;
   }
   return vect;
}

bool test( vector<unsigned>& v ) {    // финальный контроль монотонности
   bool dir = v[ 0 ] < v[ 1 ];        // порядок сортировки
   vector<unsigned>::iterator i = v.begin();
   while( true  ) {
      auto j = i;
      if( ++j == v.end() ) break;
      if( ( *i <= *j ) != dir ) return false;
      i++; 
   }
   return true;
}

typedef void (sort_func)( vector<unsigned>& );
// предварительные объявления функций сортировки:
sort_func sort1, sort2, sort3, sort4, sort5, sort6;
sort_func* variants[] = {
   sort1, sort2, sort3, sort4, sort5, sort6
};

ostream& operator <<( ostream& stream, vector<unsigned>& v ) { // отладка-контроль
   stream << "[ ";
   for( auto x : v ) stream << x << " ";
   return stream << "]";
}

int main( int argc, char *argv[] ) {
   if( argc < 3 ) return 1;
   char *parm = argv[ 1 ];    // длина и порядок тест-последовательности
   char mode = *parm == '-' || *parm == '+' || *parm == '?' ? *parm++ : '-'; 
   int size = atoi( parm );
   if( size <= 0 ) return 2;
   int debug = 0;
   parm = argv[ 2 ];          // вариант и уровень отладки
   while( '+' == *parm ) debug++, parm++;
   unsigned var = atoi( parm );
   if( var > 0 && var <= sizeof( variants ) / sizeof( variants[ 0 ] ) )
      var--;
   else return 3;
   vector<unsigned> vect = create( size, mode ); 
   if( debug ) cout << vect << endl;
   variants[ var ]( vect );
   if( debug ) cout << vect << endl; 
   else 
      cout << ( test( vect ) ? "OK" : "not OK" ) << endl;
}
//--------------------------------------------------------------------------------------------
// Быстрая сортировка. Сложность в среднем O( n*log(n) ), но в худшем случае O( n*n )
void sort1( vector<unsigned>& v ) {
   sort( v.begin(), v.end() );
};
//--------------------------------------------------------------------------------------------
bool sort_function( unsigned f, unsigned s ) { return f > s; }

void sort2( vector<unsigned>& v ) { // использование функции как предикакта
   sort( v.begin(), v.end(), sort_function );
};
//--------------------------------------------------------------------------------------------
struct sort_class {   // функтор сравнения
   bool operator()( unsigned f, unsigned s ) { return f > s; }
};

void sort3( vector<unsigned>& v ) {
   sort( v.begin(), v.end(), sort_class() );
};
//--------------------------------------------------------------------------------------------
// Сортировка подпоследователности. Гарантированная сложность O( n*log(n) ) в любом случае.
// Обычно сортировка в куче выполняется в 2-5 раз медленнее быстрой сортировки sort(). 
void sort4( vector<unsigned>& v ) {
   partial_sort( v.begin(), v.end(), v.end() );
};
//--------------------------------------------------------------------------------------------
// Сортировка слиянием. Сложность O( n*log(n) ) или O( n*log(n)*log(n) ), если без дополнительной памяти 
void sort5( vector<unsigned>& v ) {
   stable_sort( v.begin(), v.end() );
};
//--------------------------------------------------------------------------------------------
// Сортировка в куче (heap) - вызывают функции, непосредственно работающие с кучей 
// (то есть с бинарным деревом, используемым в реализации этих алгоритмов). 
// Сложность O( n+n*log(n) )
void sort6( vector<unsigned>& v ) {
   make_heap( v.begin(), v.end() );
   sort_heap( v.begin(), v.end() );
};
//--------------------------------------------------------------------------------------------
